import { StyleProperties, Style, ImageProps, InputProps, TextProps, UploadProps, ViewProps, AsProp, Length, SxProp } from "@compositive/primitives";
import { Color as PrimitiveColor } from "@compositive/primitives";
import { Style as StaticStyle } from "@compositive/primitives";
import { Style as StaticStyle$0 } from "@compositive/primitives";
import { Style as StaticStyle$1 } from "@compositive/primitives";
import { NullablePartial, Merge, HKT } from "@compositive/commons-types";
import React from "react";
import { ReactElement, ReactNode, SyntheticEvent, RefAttributes, DependencyList } from "react";
import { InferType, Equals } from "@compositive/commons-predicates";
import { AttributeDefinitions, AttributeValueDefinition, Evaluator, ChainEvaluator } from "@compositive/commons-spec";
import { Indexed, Sequence } from "@compositive/commons-sequences";
declare class SurfaceParts<T extends UnknownTheme> {
    #private;
    get prependedComponents(): ReactElement[];
    prependComponent(component: ReactElement): SurfaceParts<T>;
    get appendedComponents(): ReactElement[];
    appendComponent(component: ReactElement): SurfaceParts<T>;
    get innerStyles(): ThemedSxProp<T>;
    addInnerStyles(styles: ThemedSxProp<T>): SurfaceParts<T>;
    get outerStyles(): ThemedSxProp<T>;
    addOuterStyles(styles: ThemedSxProp<T>): SurfaceParts<T>;
}
interface View {
    <T extends UnknownTheme>(props: RefAttributes<HTMLElement> & ViewProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace View {
    type Props<T extends UnknownTheme> = ViewProps<ThemedSxPropItem<T>>;
}
declare const View: View;
interface Text {
    <T extends UnknownTheme>(props: RefAttributes<HTMLElement> & TextProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Text {
    type Props<T extends UnknownTheme> = TextProps<ThemedSxPropItem<T>>;
}
declare const Text: Text;
interface Image {
    <T extends UnknownTheme>(props: ImageProps<ThemedSxPropItem<T>> & RefAttributes<HTMLImageElement>): JSX.Element;
}
declare namespace Image {
    type Props<T extends UnknownTheme> = ImageProps<ThemedSxPropItem<T>>;
}
declare const Image: Image;
interface Input {
    <T extends UnknownTheme>(props: InputProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Input {
    type Props<T extends UnknownTheme> = InputProps<ThemedSxPropItem<T>>;
}
declare const Input: Input;
interface Upload {
    <T extends UnknownTheme>(props: RefAttributes<HTMLInputElement> & UploadProps<ThemedSxPropItem<T>>): JSX.Element;
}
declare namespace Upload {
    type Props<T extends UnknownTheme> = UploadProps<ThemedSxPropItem<T>>;
}
declare const Upload: Upload;
declare namespace Surface {
    type Attributes = {
        shape: Surface.Element<"shape">;
        corners?: Surface.Element<"corners">;
        border?: Surface.Element<"border">;
        background?: Surface.Element<"background">;
        foreground?: ColorPicker | Shade; // FIXME: What to do about foreground?
        shadow?: Surface.Element<"shadow">;
    };
    interface Element<E extends Surface.ElementType = Surface.ElementType> {
        readonly type: E;
        process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
            SurfaceParts<T>,
            Surface.Element[]
        ];
    }
    type ElementType = Exclude<keyof Surface.Attributes, "foreground">;
    type ElementChain = Exclude<Surface.Attributes[Surface.ElementType], undefined>[];
    type StateProp = {
        state: "default" | "disabled" | "hovered" | "pressed";
    };
    type Props<T extends UnknownTheme> = Partial<Surface.StateProp> & {
        children?: ReactNode;
        sx?: ThemedSxProp<T>;
        // TODO: This is an escape hatch, figure out a better solution
        innerSx?: ThemedSxProp<T>;
        as?: AsProp;
        onHoverStart?: (e: SyntheticEvent) => void;
        onHoverEnd?: (e: SyntheticEvent) => void;
        onPress?: (e: SyntheticEvent) => void;
        onPressIn?: (e: SyntheticEvent) => void;
        onPressOut?: (e: SyntheticEvent) => void;
    };
    type ComponentType<P = {}> = <T extends UnknownTheme>(props: P & Surface.Props<T>) => ReactElement;
}
declare const SurfaceSpec_base: import("@compositive/commons-spec").SpecClass<Surface.Attributes, Surface.StateProp>;
declare class SurfaceSpec<TVariants extends Surface.StateProp = Surface.StateProp> extends SurfaceSpec_base {
    toComponent(): Surface.ComponentType<Partial<TVariants>>;
    private _getSurfaceParts;
}
declare const SolidFill_base: import("@compositive/commons-spec").SpecClassFromPredicates<{
    shade: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<Shade>;
}, Record<never, string>>;
declare class SolidFill extends SolidFill_base implements Surface.Element<"background"> {
    readonly type = "background";
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    private _getStyle;
}
declare const SolidBorder_base: import("@compositive/commons-spec").SpecClassFromPredicates<{
    width: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    shade: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<Shade>;
}, Record<never, string>>;
declare class SolidBorder extends SolidBorder_base implements Surface.Element<"border"> {
    readonly type = "border";
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    private _getStyle;
}
declare const isRoundedCornersAttributes: import("@compositive/commons-predicates").ObjectWithPropertiesPredicate<{
    radius: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    topLeft: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    topRight: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    bottomLeft: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    bottomRight: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
}>;
type RoundedCornersAttributes = InferType<typeof isRoundedCornersAttributes>;
declare const RoundedCorners_base: import("@compositive/commons-spec").SpecClass<import("@compositive/commons-predicates/dist/createPredicate-490389e8").InferRecordType<{
    radius: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    topLeft: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    topRight: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    bottomLeft: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    bottomRight: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
}>, Record<never, string>>;
declare class RoundedCorners extends RoundedCorners_base implements Surface.Element<"corners"> {
    readonly type = "corners";
    constructor(radiusOrAttribs: RoundedCornersAttributes | number, previousSpec?: RoundedCorners);
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    // FIXME: Use the other attributes
    private _getStyle;
}
declare const DropShadow_base: import("@compositive/commons-spec").SpecClassFromPredicates<{
    elevation: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number>;
    tone: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<Shade>;
}, Record<never, string>>;
declare class DropShadow extends DropShadow_base implements Surface.Element<"shadow"> {
    readonly type = "shadow";
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
}
type RectangleAttributes = {
    padding?: number;
};
declare const Rectangle_base: import("@compositive/commons-spec").SpecClassFromPredicates<{
    padding: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
}, Record<never, string>>;
declare class Rectangle extends Rectangle_base implements Surface.Element<"shape"> {
    readonly type = "shape";
    constructor(attribs?: RectangleAttributes, previousSpec?: Rectangle);
    process<T extends UnknownTheme>(parts: SurfaceParts<T>, chain: Surface.Element[]): [
        SurfaceParts<T>,
        Surface.Element[]
    ];
    private _getStyle;
}
declare function useSurface<T extends AnyTheme>(names: readonly (string | symbol)[]): Surface.ComponentType;
declare enum FontStyle {
    Normal = "normal",
    Italic = "italic",
    Oblique = "oblique"
}
type FontWeightMap = {
    Thin: 100;
    ExtraLight: 200;
    Light: 300;
    Regular: 400;
    Medium: 500;
    SemiBold: 600;
    Bold: 700;
    ExtraBold: 800;
    Black: 900;
};
declare class FontWeight<N extends keyof FontWeightMap = keyof FontWeightMap> {
    #private;
    static readonly Thin: FontWeight<"Thin">;
    static readonly ExtraLight: FontWeight<"ExtraLight">;
    static readonly Light: FontWeight<"Light">;
    static readonly Regular: FontWeight<"Regular">;
    static readonly Medium: FontWeight<"Medium">;
    static readonly SemiBold: FontWeight<"SemiBold">;
    static readonly Bold: FontWeight<"Bold">;
    static readonly ExtraBold: FontWeight<"ExtraBold">;
    static readonly Black: FontWeight<"Black">;
    static for(weight: number): FontWeight;
    readonly value: FontWeightMap[N];
    readonly name: N;
    private constructor();
    protected get index(): number;
    get bolder(): FontWeight | undefined;
    get lighter(): FontWeight | undefined;
    get fallbacks(): Readonly<FontWeight[]>;
    isLighterThan(other: FontWeight): boolean;
    isBolderThan(other: FontWeight): boolean;
    toString(): string;
}
declare class FontDesignation<W extends FontWeight = FontWeight, S extends FontStyle = FontStyle> {
    static fromFvd(fvd: string): FontDesignation<FontWeight<any>>;
    static for<W extends FontWeight<any>, S extends FontStyle>(weight: W, style: S): FontDesignation<W, S>;
    readonly weight: W;
    readonly style: S;
    private constructor();
    get fallbacks(): FontDesignation<any, any>[];
    get fvd(): string;
    get googleFontsDescription(): string;
    toString(): string;
    toHumanString(): string;
}
interface FontCatalog<F extends FontFamily> {
    getAll(): Promise<F[]>;
    get(name: string): Promise<F>;
}
interface FontLoader {
    loadMany(fonts: Iterable<Font>): Promise<void>;
    load(font: Font): Promise<void>;
    isLoaded(font: Font): boolean;
}
declare enum FontType {
    Cursive = "cursive",
    Fantasy = "fantasy",
    Monospace = "monospace",
    SansSerif = "sans-serif",
    Serif = "serif"
}
type WithThinFont = {
    readonly Thin: Font;
};
type WithThinItalicFont = {
    readonly ThinItalic: Font;
};
type WithThinObliqueFont = {
    readonly ThinOblique: Font;
};
type WithExtraLightFont = {
    readonly ExtraLight: Font;
};
type WithExtraLightItalicFont = {
    readonly ExtraLightItalic: Font;
};
type WithExtraLightObliqueFont = {
    readonly ExtraLightOblique: Font;
};
type WithLightFont = {
    readonly Light: Font;
};
type WithLightItalicFont = {
    readonly LightItalic: Font;
};
type WithLightObliqueFont = {
    readonly LightOblique: Font;
};
type WithRegularFont = {
    readonly Regular: Font;
};
type WithRegularItalicFont = {
    readonly RegularItalic: Font;
};
type WithRegularObliqueFont = {
    readonly RegularOblique: Font;
};
type WithMediumFont = {
    readonly Medium: Font;
};
type WithMediumItalicFont = {
    readonly MediumItalic: Font;
};
type WithMediumObliqueFont = {
    readonly MediumOblique: Font;
};
type WithSemiBoldItalicFont = {
    readonly SemiBoldItalic: Font;
};
type WithSemiBoldFont = {
    readonly SemiBold: Font;
};
type WithSemiBoldObliqueFont = {
    readonly SemiBoldOblique: Font;
};
type WithBoldItalicFont = {
    readonly BoldItalic: Font;
};
type WithBoldFont = {
    readonly Bold: Font;
};
type WithBoldObliqueFont = {
    readonly BoldOblique: Font;
};
type WithExtraBoldItalicFont = {
    readonly ExtraBoldItalic: Font;
};
type WithExtraBoldFont = {
    readonly ExtraBold: Font;
};
type WithExtraBoldObliqueFont = {
    readonly ExtraBoldOblique: Font;
};
type WithBlackFont = {
    readonly Black: Font;
};
type WithBlackItalicFont = {
    readonly BlackItalic: Font;
};
type WithBlackObliqueFont = {
    readonly BlackOblique: Font;
};
declare abstract class FontFamily {
    #private;
    readonly name: string;
    readonly fallback: (FontFamily | string)[];
    readonly fontType: FontType;
    constructor(name: string, fontType: FontType, fallback?: (FontFamily | string)[]);
    abstract get fontLoader(): FontLoader;
    abstract get fontCatalog(): FontCatalog<FontFamily>;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Normal): WithThinFont & this;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Italic): WithThinItalicFont & this;
    withFont(w: FontWeight<"Thin">, s: FontStyle.Oblique): WithThinObliqueFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Normal): WithExtraLightFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Italic): WithExtraLightItalicFont & this;
    withFont(w: FontWeight<"ExtraLight">, s: FontStyle.Oblique): WithExtraLightObliqueFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Normal): WithLightFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Italic): WithLightItalicFont & this;
    withFont(w: FontWeight<"Light">, s: FontStyle.Oblique): WithLightObliqueFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Normal): WithRegularFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Italic): WithRegularItalicFont & this;
    withFont(w: FontWeight<"Regular">, s: FontStyle.Oblique): WithRegularObliqueFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Normal): WithMediumFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Italic): WithMediumItalicFont & this;
    withFont(w: FontWeight<"Medium">, s: FontStyle.Oblique): WithMediumObliqueFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Normal): WithSemiBoldFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Italic): WithSemiBoldItalicFont & this;
    withFont(w: FontWeight<"SemiBold">, s: FontStyle.Oblique): WithSemiBoldObliqueFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Normal): WithBoldFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Italic): WithBoldItalicFont & this;
    withFont(w: FontWeight<"Bold">, s: FontStyle.Oblique): WithBoldObliqueFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Normal): WithExtraBoldFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Italic): WithExtraBoldItalicFont & this;
    withFont(w: FontWeight<"ExtraBold">, s: FontStyle.Oblique): WithExtraBoldObliqueFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Normal): WithBlackFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Italic): WithBlackItalicFont & this;
    withFont(w: FontWeight<"Black">, s: FontStyle.Oblique): WithBlackObliqueFont & this;
    /**
     * List of the fonts belonging to this family.
     */
    get fonts(): Readonly<Font[]>;
    get weights(): Readonly<FontWeight<any>[]>;
    get styles(): Readonly<FontStyle[]>;
    getFont(designation: FontDesignation): Font | undefined;
    getFont(weight: FontWeight, style: FontStyle): Font | undefined;
    getClosestFont(weight: FontWeight, style: FontStyle): Font | undefined;
    toString(): string;
}
declare enum TextTransform {
    None = "none",
    Uppercase = "uppercase",
    Lowercase = "lowercase",
    Capitalize = "capitalize"
}
declare class Font {
    readonly family: FontFamily;
    readonly weight: FontWeight;
    readonly style: FontStyle;
    readonly fontLoader: FontLoader;
    constructor(family: FontFamily, weight: FontWeight, style: FontStyle, fontLoader: FontLoader);
    withSize(size: AttributeValueDefinition<FontSpec["attributes"], "size">): FontSpec;
    with(attributeName: "size", value: AttributeValueDefinition<FontSpec["attributes"], "size">): FontSpec;
    with(values: Omit<AttributeDefinitions<FontSpec["attributes"]>, "font">): FontSpec;
    get designation(): FontDesignation;
    get bolder(): Font;
    get lighter(): Font;
    get italic(): Font;
    get normal(): Font;
    load(): Promise<void>;
    isLoaded(): boolean;
    toString(): string;
}
declare const FontSpec_base: import("@compositive/commons-spec").SpecClassFromPredicates<{
    size: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number>;
    lineHeight: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    font: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<Font>;
    letterSpacing: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<number | undefined>;
    textTransform: import("@compositive/commons-predicates/dist/createPredicate-490389e8").Predicate<TextTransform | undefined>;
}, Record<never, string>>;
declare class FontSpec extends FontSpec_base implements ConvertibleToStyle<never> {
    get allFonts(): Font[];
    toStaticStyle(): StaticStyle;
}
declare function useTypography<T extends AnyTheme = DesignSystemThemes>(knownSpecNames: Theme.ReferenceCascade<T, "typography">): FontSpec;
declare function useTypography<_T extends AnyTheme = DesignSystemThemes>(specNames: readonly (string | symbol)[]): FontSpec | undefined;
declare abstract class Theme<A extends Theme.Attributes, V extends Theme.Variants> {
    get allFonts(): Font[];
    loadAllFonts(): Promise<void>;
    areFontsLoaded(): boolean;
    abstract using(activeVariants: NullablePartial<V>): Theme<A, V>;
    abstract get surfaces(): A["surfaces"];
    abstract get typography(): A["typography"];
    abstract get metrics(): A["metrics"];
    abstract get palette(): A["palette"];
    abstract get colorPicks(): A["colorPicks"];
    abstract get variants(): Theme.VariantOptions<V>;
}
type AnyTheme = Theme<Theme.Open<Theme.Attributes>, Theme.Variants>;
type UnknownTheme = Theme<Theme.Attributes, Theme.NoVariants>;
type PartialRecord<T> = Readonly<Record<string | symbol, T | undefined>>;
declare namespace Theme {
    type Attributes = {
        readonly metrics: {
            readonly grid: number;
        };
        readonly palette: {
            readonly primary: ColorScale;
            readonly neutral: ColorScale;
            readonly info: ColorScale;
            readonly danger: ColorScale;
            readonly success: ColorScale;
            readonly warning: ColorScale;
        };
        readonly colorPicks: {
            readonly background: ColorPicker;
            readonly foreground: ColorPicker;
        };
        readonly surfaces: {
            readonly default: SurfaceSpec;
        };
        readonly typography: {
            readonly default: FontSpec;
        };
    };
    type Open<A extends Attributes> = {
        readonly metrics: A["metrics"];
        readonly palette: A["palette"] & PartialRecord<ColorScale>;
        readonly colorPicks: A["colorPicks"] & PartialRecord<ColorPicker>;
        readonly surfaces: A["surfaces"] & PartialRecord<SurfaceSpec>;
        readonly typography: A["typography"] & PartialRecord<FontSpec>;
    };
    type AttributeExtraKeys = Partial<Record<Exclude<keyof Attributes, "metrics">, string | symbol>>;
    type AttributesWithExtraKeys<A extends Attributes, K extends AttributeExtraKeys> = {
        readonly [I in keyof A]: A[I] & (I extends keyof K ? undefined extends K[I] ? unknown : {
            readonly [J in K[I] & (string | symbol)]: NonNullable<A[I][keyof A[I]]>;
        } : unknown);
    };
    type OnlyKnownAttributes<T extends Attributes> = {
        readonly [K in keyof T]: {
            readonly [J in keyof T[K] as undefined extends T[K][J] ? never : J]: T[K][J];
        };
    };
    type Variants = Readonly<Record<string, string>>;
    type InferVariants<T extends Theme<any, any>> = [
        T
    ] extends [
        never
    ] ? Theme.NoVariants : T extends Theme<any, infer V> ? V : Theme.NoVariants;
    type VariantOptions<V extends Theme.Variants> = {
        readonly [K in keyof V]: readonly V[K][];
    };
    type NoVariants = Record<never, never>;
    type Reference<T extends AnyTheme, K extends keyof T> = T extends any ? keyof T[K] & (string | symbol) : never;
    type ReferenceCascade<T extends AnyTheme, K extends keyof T> = readonly [
        ...(string | symbol)[],
        keyof T[K] & (string | symbol)
    ] | (keyof T[K] & (string | symbol));
}
interface AvailableThemes {
}
type DesignSystemThemes = [
    AvailableThemes[keyof AvailableThemes]
] extends [
    never
] ? UnknownTheme : AvailableThemes[keyof AvailableThemes];
// FIXME: The previous type in the definition thunks for non-existing attributes should be "never" instead of "unknown"
// FIXME: Variant options should merge, not be replaced
/**
 * A {@link ComputedTheme} is a {@link Theme} implementation that allows for its values to be derived through the use of
 * pure functions and derivation rules.
 *
 * It's possible to create new themes from scratch, or through the extension of other themes,
 * by overriding only some of the values.
 */
declare class ComputedTheme<A extends Theme.Attributes, V extends Theme.Variants> extends Theme<A, V> {
    #private;
    static with<D extends ComputedTheme.FullAttributeDefinition>(definition: D): ComputedTheme.InitiallyDefinedBy<D>;
    private constructor();
    using(activeVariants: NullablePartial<V>): Theme<A, V>;
    [Equals](other: unknown): boolean;
    with<D extends ComputedTheme.OpenPartialAttributeDefinition<A>>(definition: D): ComputedTheme.WithExtraDefinition<A, D>;
    when<N extends string, K extends string>(variantName: N, options: Readonly<Record<K, ComputedTheme.ClosedPartialAttributeDefinition<A>>>): ComputedTheme.WithExtraVariant<A, V, N, K>;
    get surfaces(): A["surfaces"];
    get typography(): A["typography"];
    get metrics(): A["metrics"];
    get palette(): A["palette"];
    get colorPicks(): A["colorPicks"];
    get variants(): Theme.VariantOptions<V>;
}
type AnyComputedTheme = ComputedTheme<Theme.Open<Theme.Attributes>, Theme.Variants>;
type UnknownComputedTheme = ComputedTheme<Theme.Attributes, Theme.NoVariants>;
declare namespace ComputedTheme {
    type InitiallyDefinedBy<D extends OpenAttributeDefinition<any>> = WithExtraDefinition<Theme.Attributes, D>;
    type WithExtraDefinition<A extends Theme.Attributes, D extends OpenPartialAttributeDefinition<any>> = ComputedTheme<AttributesWithExtraDefinitions<A, D>, Theme.NoVariants>;
    type WithExtraVariant<A extends Theme.Attributes, V extends Theme.Variants, N extends string, K extends string> = ComputedTheme<A, Merge<V, {
        readonly [I in N]: K;
    }>>;
    type OpenPartialAttributes = {
        readonly [K in keyof Theme.Attributes]?: Readonly<Record<string, Theme.Attributes[K][keyof Theme.Attributes[K]] | undefined>>;
    };
    type ClosedPartialAttributes<A extends Theme.Attributes> = {
        readonly [K in string & keyof A]?: {
            readonly [J in string & keyof A[K]]?: A[K][J];
        };
    };
    type FullAttributeDefinition = OpenAttributeDefinition<{}>;
    type OpenPartialAttributeDefinition<P extends Theme.Attributes> = Evaluator.AttributeDefinition<1, OpenPartialAttributes, P, Theme.Open<P>>;
    type ClosedPartialAttributeDefinition<P extends Theme.Attributes> = ChainEvaluator.PartialAttributeDefinition<1, P, P>;
    type OpenAttributeDefinition<P extends Evaluator.AttributeTypeRecord<1>> = Evaluator.AttributeDefinition<1, Theme.Open<Theme.Attributes>, P, Theme.Open<Theme.Attributes>>;
    type DefinedKeys<D extends Evaluator.AttributeTypeRecord<1>> = {
        [I in keyof D]: string & keyof D[I];
    };
    type AttributesWithExtraDefinitions<A extends Theme.Attributes, D extends Evaluator.AttributeTypeRecord<1>> = Theme.AttributesWithExtraKeys<A, DefinedKeys<D>>;
}
declare const units: (multiplier?: number) => <T extends AnyTheme>(theme: T) => Length;
declare const u: (multiplier?: number) => <T extends AnyTheme>(theme: T) => Length;
declare namespace ThemeConsumer {
    type Props<T extends UnknownTheme> = {
        children: (value: T) => ReactNode;
    };
}
declare const ThemeConsumer: <T extends UnknownTheme = UnknownTheme>({ children }: ThemeConsumer.Props<T>) => ReactElement;
declare namespace ThemeProvider {
    type Props<T extends UnknownTheme> = NullablePartial<Theme.InferVariants<T>> & {
        theme: T;
        children: React.ReactNode;
    };
}
declare const ThemeProvider: <T extends UnknownTheme>({ theme, children, ...variants }: ThemeProvider.Props<T>) => ReactElement;
declare const useTheme: <T extends UnknownTheme = UnknownTheme>() => T;
type ThemedStyleProperties<T extends UnknownTheme> = {
    [X in keyof StyleProperties]: StyleProperties[X] | ((theme: T) => StyleProperties[X]);
};
type ThemedStylePropertiesGenerator<T extends UnknownTheme> = ThemedStyleProperties<T> | ((theme: T) => ThemedStyleProperties<T>);
declare const createThemeSxProcessor: <T extends UnknownTheme>(theme: T) => (style: ThemedStyleProperties<T>) => Style | StyleProperties;
declare const resolveThemedStyleProperties: <T extends UnknownTheme>(themedStyleProperties: ThemedStylePropertiesGenerator<T>, theme: T) => StyleProperties;
declare const isConvertibleToStyle: <T extends UnknownTheme>(value: unknown) => value is ConvertibleToStyle<T>;
interface ConvertibleToStyle<T extends UnknownTheme> {
    toStaticStyle(theme: T): Style;
}
type ThemedSxPropItem<T extends UnknownTheme> = ConvertibleToStyle<T> | Style | StyleProperties | ThemedStyleProperties<T>;
type ThemedSxProp<T extends UnknownTheme> = SxProp<ThemedSxPropItem<T>>;
declare class ThemedStyle<T extends UnknownTheme> implements ConvertibleToStyle<T> {
    #private;
    // TODO: Copying this static method here is not the best API design decision.
    // It would be better to find a better structure to separate "Style" manipulation from
    // the "SxProp" manipulation
    static concat<T extends UnknownTheme>(...sxProps: ThemedSxProp<T>[]): ThemedSxProp<T>;
    constructor(themedStyleProperties: ThemedStylePropertiesGenerator<T>);
    toStaticStyle(theme: T): StaticStyle$0;
}
declare module ThemedStyleWrapper {
    export { ThemedStyle };
}
import Style$0 = ThemedStyleWrapper.ThemedStyle;
declare module StyleWrapper {
    export { Style$0 as Style };
}
import ThemedStyle$0 = StyleWrapper.Style;
declare const useStaticStyle: (rules: () => StyleProperties, deps: DependencyList) => Style;
declare const useThemedStyle: <T extends UnknownTheme = UnknownTheme>(rules: () => ThemedStyleProperties<T>, deps: DependencyList) => ThemedStyle$0<T>;
declare class Color implements ConvertibleToStyle<never> {
    #private;
    static fromBinary([r, g, b]: Uint8Array): Color;
    constructor(colorValue: ColorLikeValue);
    asForeground(): StaticStyle$1;
    asBackground(): StaticStyle$1;
    toStaticStyle(): StaticStyle$1;
    get hex(): HexColorString;
    get r(): number;
    get g(): number;
    get b(): number;
    get h(): number;
    get s(): number;
    get l(): number;
    get a(): number;
    get rgb(): [
        red: number,
        green: number,
        blue: number
    ];
    get hsl(): [
        hue: number,
        saturation: number,
        lightness: number
    ];
    alpha(alpha: number): Color;
    toString(): string;
    toBinary(): Uint8Array;
}
type HexColorString = `#${string}`;
type ColorLikeValue = Color | HexColorString | PrimitiveColor;
declare class Shade extends Color {
    readonly colorScale: ColorScale;
    readonly index: number;
    constructor(colorScale: ColorScale, index: number, colorValue: ColorLikeValue);
    transposeOn(otherScale: ColorScale): Shade | undefined;
    backward(steps?: number, outOfBoundariesMethod?: "none"): Shade | undefined;
    backward(steps: number, outOfBoundariesMethod: "clamp" | "wrap"): Shade;
    frontward(steps?: number, outOfBoundariesMethod?: "none"): Shade | undefined;
    frontward(steps: number, outOfBoundariesMethod: "clamp" | "wrap"): Shade;
    [Equals](other: unknown): boolean;
}
declare namespace ColorScale {
    type OutOfBoundsMethod = "clamp" | "none" | "wrap";
}
declare namespace ColorScale {
    interface Type extends HKT {
        readonly type: FixedColorScale;
    }
}
declare abstract class ColorScale extends Sequence<Shade> implements Indexed<number, Shade | undefined>, HKT.Holder {
    hktForSameElementKind: ColorScale.Type;
    constructor();
    get(key: number, OutOfBoundsMethod?: "none"): Shade | undefined;
    get(key: number, OutOfBoundsMethod: "clamp" | "wrap"): Shade;
    get(key: number, outOfBoundsMethod: ColorScale.OutOfBoundsMethod): Shade | undefined;
    has(key: number): boolean;
    get length(): number;
    get backmost(): Shade;
    get midpoint(): Shade;
    get frontmost(): Shade;
    filter(cb: (value: Shade, iteration: number) => boolean): ColorScale;
    [Equals](other: unknown): boolean;
    abstract get shades(): Shade[];
    abstract reversed(): ColorScale;
    protected buildNextWithSameElementKind<H extends HKT | HKT.Holder>(source: Iterable<Shade> | (() => Iterator<Shade>)): HKT.Kind<H, [
        Shade
    ]>;
}
declare class FixedColorScale extends ColorScale {
    readonly shades: Shade[];
    constructor(colors: Iterable<ColorLikeValue> | (() => Iterator<ColorLikeValue>));
    reversed(): FixedColorScale;
}
interface ColorPicker {
    pickColor(palette: ColorScale, background?: Shade): Shade;
}
declare const ColorPicker: {
    create(fn: ColorPicker["pickColor"]): ColorPicker;
};
declare namespace ColorContext {
    type ColorPick<T extends UnknownTheme = DesignSystemThemes> = ColorPicker | Shade | Theme.Reference<T, "colorPicks">;
    type Output = {
        foreground: Shade;
        background: Shade;
        palette: ColorScale;
    };
    type Input<T extends UnknownTheme = DesignSystemThemes> = {
        foreground: ColorPick<T>;
        background: ColorPick<T>;
        palette: Theme.Reference<T, "palette">;
    };
}
declare namespace ColorProvider {
    type Props<T extends UnknownTheme = DesignSystemThemes> = Partial<ColorContext.Input<T>> & {
        children: ReactNode;
    };
}
declare const ColorProvider: <T extends UnknownTheme = UnknownTheme>(props: ColorProvider.Props<T>) => ReactElement;
declare namespace ColorConsumer {
    type Props = {
        children: (values: ColorContext.Output) => ReactElement;
    };
}
declare const ColorConsumer: ({ children }: ColorConsumer.Props) => ReactElement;
declare const useCurrentForeground: () => Shade;
declare const useCurrentBackground: () => Shade;
declare const useCurrentPalette: () => ColorScale;
declare const transparent: Color;
// Red colors
declare const mediumVioletRed: Color;
declare const deepPink: Color;
declare const paleVioletRed: Color;
declare const hotPink: Color;
declare const lightPink: Color;
declare const pink: Color;
declare const darkRed: Color;
declare const red: Color;
declare const firebrick: Color;
declare const crimson: Color;
declare const indianRed: Color;
declare const lightCoral: Color;
declare const salmon: Color;
declare const darkSalmon: Color;
declare const lightSalmon: Color;
// Orange colors
declare const orangeRed: Color;
declare const tomato: Color;
declare const darkOrange: Color;
declare const coral: Color;
declare const orange: Color;
// Yellow colors
declare const darkKhaki: Color;
declare const gold: Color;
declare const khaki: Color;
declare const peachPuff: Color;
declare const yellow: Color;
declare const paleGoldenrod: Color;
declare const moccasin: Color;
declare const papayaWhip: Color;
declare const lightGoldenrodYellow: Color;
declare const lemonChiffon: Color;
declare const lightYellow: Color;
// Brown colors
declare const maroon: Color;
declare const brown: Color;
declare const saddleBrown: Color;
declare const sienna: Color;
declare const chocolate: Color;
declare const darkGoldenrod: Color;
declare const peru: Color;
declare const rosyBrown: Color;
declare const goldenrod: Color;
declare const sandyBrown: Color;
declare const tan: Color;
declare const burlywood: Color;
declare const wheat: Color;
declare const navajoWhite: Color;
declare const bisque: Color;
declare const blanchedAlmond: Color;
declare const cornsilk: Color;
// Green colors
declare const darkGreen: Color;
declare const green: Color;
declare const darkOliveGreen: Color;
declare const forestGreen: Color;
declare const seaGreen: Color;
declare const olive: Color;
declare const oliveDrab: Color;
declare const mediumSeaGreen: Color;
declare const limeGreen: Color;
declare const lime: Color;
declare const springGreen: Color;
declare const mediumSpringGreen: Color;
declare const darkSeaGreen: Color;
declare const mediumAquamarine: Color;
declare const yellowGreen: Color;
declare const lawnGreen: Color;
declare const chartreuse: Color;
declare const lightGreen: Color;
declare const greenYellow: Color;
declare const paleGreen: Color;
// Cyan colors
declare const teal: Color;
declare const darkCyan: Color;
declare const lightSeaGreen: Color;
declare const cadetBlue: Color;
declare const darkTurquoise: Color;
declare const mediumTurquoise: Color;
declare const turquoise: Color;
declare const aqua: Color;
declare const cyan: Color;
declare const aquamarine: Color;
declare const paleTurquoise: Color;
declare const lightCyan: Color;
// Blue colors
declare const navy: Color;
declare const darkBlue: Color;
declare const mediumBlue: Color;
declare const blue: Color;
declare const midnightBlue: Color;
declare const royalBlue: Color;
declare const steelBlue: Color;
declare const dodgerBlue: Color;
declare const deepSkyBlue: Color;
declare const cornflowerBlue: Color;
declare const skyBlue: Color;
declare const lightSkyBlue: Color;
declare const lightSteelBlue: Color;
declare const lightBlue: Color;
declare const powderBlue: Color;
// Purple, violet, and magenta colors
declare const indigo: Color;
declare const purple: Color;
declare const darkMagenta: Color;
declare const darkViolet: Color;
declare const darkSlateBlue: Color;
declare const blueViolet: Color;
declare const darkOrchid: Color;
declare const fuchsia: Color;
declare const magenta: Color;
declare const slateBlue: Color;
declare const mediumSlateBlue: Color;
declare const mediumOrchid: Color;
declare const mediumPurple: Color;
declare const orchid: Color;
declare const violet: Color;
declare const plum: Color;
declare const thistle: Color;
declare const lavender: Color;
// White colors
declare const mistyRose: Color;
declare const antiqueWhite: Color;
declare const linen: Color;
declare const beige: Color;
declare const whiteSmoke: Color;
declare const lavenderBlush: Color;
declare const oldLace: Color;
declare const aliceBlue: Color;
declare const seashell: Color;
declare const ghostWhite: Color;
declare const honeydew: Color;
declare const floralWhite: Color;
declare const azure: Color;
declare const mintCream: Color;
declare const snow: Color;
declare const ivory: Color;
declare const white: Color;
// Gray and black colors
declare const black: Color;
declare const darkSlateGray: Color;
declare const dimGray: Color;
declare const slateGray: Color;
declare const gray: Color;
declare const lightSlateGray: Color;
declare const darkGray: Color;
declare const silver: Color;
declare const lightGray: Color;
declare const gainsboro: Color;
declare enum ContrastLevel {
    /** Less than WCAG 3:1 contrast ratio */
    INSUFFICIENT = 0,
    /** Equivalent to WCAG A (3..4.5:1) level */
    MINIMUM = 1,
    /** Equivalent to WCAG AA (4.5..7:1) level */
    GOOD = 2,
    /** Equivalent to WCAG AAA (\>= 7:1) level */
    HIGH = 3
}
type Contrast = {
    readonly value: number;
    readonly level: ContrastLevel;
    toString(): string;
};
type ContrastChecker = (background: Color, foreground: Color) => Contrast;
declare const getWCAGContrast: ContrastChecker;
declare const rgb: (r: number, g: number, b: number) => Color;
declare const rgba: (r: number, g: number, b: number, a: number) => Color;
declare const hsl: (h: number, s: number, l: number) => Color;
declare const hsla: (h: number, s: number, l: number, a: number) => Color;
declare const hex: (color: string) => Color;
type Easing = (x: number) => number;
type Options = {
    lightnessMin?: number;
    lightnessMax?: number;
    easing?: Easing;
};
declare class UniformLightnessColorScale extends ColorScale {
    readonly baseColor: Color;
    readonly lightnessMin: number;
    readonly lightnessMax: number;
    readonly easing: Easing;
    readonly steps: number;
    constructor(base: ColorLikeValue, steps?: number, options?: Options);
    get length(): number;
    get shades(): Shade[];
    reversed(): UniformLightnessColorScale;
}
declare const usePalette: <T extends UnknownTheme = UnknownTheme>(name: Theme.Reference<T, "palette">) => ColorScale;
/** No easing, no acceleration */
declare const linear: (t: number) => number;
/** Accelerating from zero velocity */
declare const easeInQuad: (t: number) => number;
/** Decelerating to zero velocity */
declare const easeOutQuad: (t: number) => number;
/** Acceleration until halfway, then deceleration */
declare const easeInOutQuad: (t: number) => number;
/** Accelerating from zero velocity */
declare const easeInCubic: (t: number) => number;
/** Decelerating to zero velocity */
declare const easeOutCubic: (t: number) => number;
/** Acceleration until halfway, then deceleration */
declare const easeInOutCubic: (t: number) => number;
/** Accelerating from zero velocity */
declare const easeInQuart: (t: number) => number;
/** Decelerating to zero velocity */
declare const easeOutQuart: (t: number) => number;
/** Acceleration until halfway, then deceleration */
declare const easeInOutQuart: (t: number) => number;
/** Accelerating from zero velocity */
declare const easeInQuint: (t: number) => number;
/** Decelerating to zero velocity */
declare const easeOutQuint: (t: number) => number;
/** Acceleration until halfway, then deceleration */
declare const easeInOutQuint: (t: number) => number;
/** Hyperbolic tangent, normalized to be bounded in the interval [0, 1] */
declare const tanh: (t: number) => number;
export { Color, HexColorString, ColorLikeValue, ColorContext, ColorProvider, ColorConsumer, useCurrentForeground, useCurrentBackground, useCurrentPalette, ColorPicker, transparent, mediumVioletRed, deepPink, paleVioletRed, hotPink, lightPink, pink, darkRed, red, firebrick, crimson, indianRed, lightCoral, salmon, darkSalmon, lightSalmon, orangeRed, tomato, darkOrange, coral, orange, darkKhaki, gold, khaki, peachPuff, yellow, paleGoldenrod, moccasin, papayaWhip, lightGoldenrodYellow, lemonChiffon, lightYellow, maroon, brown, saddleBrown, sienna, chocolate, darkGoldenrod, peru, rosyBrown, goldenrod, sandyBrown, tan, burlywood, wheat, navajoWhite, bisque, blanchedAlmond, cornsilk, darkGreen, green, darkOliveGreen, forestGreen, seaGreen, olive, oliveDrab, mediumSeaGreen, limeGreen, lime, springGreen, mediumSpringGreen, darkSeaGreen, mediumAquamarine, yellowGreen, lawnGreen, chartreuse, lightGreen, greenYellow, paleGreen, teal, darkCyan, lightSeaGreen, cadetBlue, darkTurquoise, mediumTurquoise, turquoise, aqua, cyan, aquamarine, paleTurquoise, lightCyan, navy, darkBlue, mediumBlue, blue, midnightBlue, royalBlue, steelBlue, dodgerBlue, deepSkyBlue, cornflowerBlue, skyBlue, lightSkyBlue, lightSteelBlue, lightBlue, powderBlue, indigo, purple, darkMagenta, darkViolet, darkSlateBlue, blueViolet, darkOrchid, fuchsia, magenta, slateBlue, mediumSlateBlue, mediumOrchid, mediumPurple, orchid, violet, plum, thistle, lavender, mistyRose, antiqueWhite, linen, beige, whiteSmoke, lavenderBlush, oldLace, aliceBlue, seashell, ghostWhite, honeydew, floralWhite, azure, mintCream, snow, ivory, white, black, darkSlateGray, dimGray, slateGray, gray, lightSlateGray, darkGray, silver, lightGray, gainsboro, ColorScale, FixedColorScale, getWCAGContrast, ContrastLevel, ContrastChecker, Shade, rgb, rgba, hsl, hsla, hex, UniformLightnessColorScale, usePalette, linear, easeInQuad, easeOutQuad, easeInOutQuad, easeInCubic, easeOutCubic, easeInOutCubic, easeInQuart, easeOutQuart, easeInOutQuart, easeInQuint, easeOutQuint, easeInOutQuint, tanh, View, Text, Image, Input, Upload, createThemeSxProcessor, resolveThemedStyleProperties, ThemedStyle as Style, ThemedStyle, isConvertibleToStyle, useStaticStyle, useThemedStyle, SolidFill, SolidBorder, RoundedCorners, DropShadow, Rectangle, SurfaceParts, Surface, SurfaceSpec, useSurface, AvailableThemes, DesignSystemThemes, ComputedTheme, AnyComputedTheme, UnknownComputedTheme, units, u, Theme, AnyTheme, UnknownTheme, ThemeConsumer, ThemeProvider, useTheme, Font, FontSpec, FontCatalog, FontDesignation, FontType, FontFamily, FontLoader, FontStyle, FontWeightMap, FontWeight, TextTransform, useTypography };
export type { Angle, AsProp, Length, StyleProperties } from "@compositive/primitives";
export type { ThemedStyleProperties, ConvertibleToStyle, ThemedSxProp, ThemedSxPropItem };
export { Style$0 as StaticStyle } from "@compositive/primitives";
//# sourceMappingURL=index.d.ts.map